/// <reference types="@vidstack/foundation/globals" />
/// <reference types="@vidstack/player/globals" />

import { AspectRatioElement } from '@vidstack/player';
import { AudioElement } from '@vidstack/player';
import { Context } from 'react';
import { ForwardRefExoticComponent } from 'react';
import { FullscreenButtonElement } from '@vidstack/player';
import { GestureElement } from '@vidstack/player';
import { HlsElement } from '@vidstack/player';
import { HTMLAttributes } from 'react';
import { MediaContext } from '@vidstack/player';
import { MediaElement } from '@vidstack/player';
import { MediaRemoteControl } from '@vidstack/player';
import { MediaSyncElement } from '@vidstack/player';
import { MediaVisibilityElement } from '@vidstack/player';
import { MuteButtonElement } from '@vidstack/player';
import type { PascalCase } from 'type-fest';
import { PlayButtonElement } from '@vidstack/player';
import { PosterElement } from '@vidstack/player';
import * as React_2 from 'react';
import { RefAttributes } from 'react';
import { RefObject } from 'react';
import { SliderElement } from '@vidstack/player';
import { SliderValueTextElement } from '@vidstack/player';
import { SliderVideoElement } from '@vidstack/player';
import { TimeElement } from '@vidstack/player';
import { TimeSliderElement } from '@vidstack/player';
import { ToggleButtonElement } from '@vidstack/player';
import { VideoElement } from '@vidstack/player';
import { VolumeSliderElement } from '@vidstack/player';

/** This element creates a container that will hold the dimensions of the desired aspect ratio. This
 container is useful for reserving space for media as it loads over the network.

 💡  If your browser matrix supports the
 [`aspect-ratio`](https://developer.mozilla.org/en-US/docs/Web/CSS/aspect-ratio) CSS property
 then you can skip using this component, and set the desired aspect ratio directly on the
 provider element.

 💡 By default it respects the browser's default aspect-ratio for media. This is not specific
 to the loaded media but instead a general setting of `2/1`. */
export declare const AspectRatio: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<AspectRatioElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "render" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "minHeight" | "maxHeight" | "ratio" | "isValidRatio" | keyof React_2.HTMLAttributes<AspectRatioElement>> & React_2.RefAttributes<AspectRatioElement>>;

/** The `<vds-audio>` element adapts the underlying `<audio>` element to satisfy the media provider
 contract, which generally involves providing a consistent API for loading, managing, and
 tracking media state.

 Most the logic for this element is contained in the `Html5MediaElement` class because both the
 `<audio>` and `<video>` elements implement the native `HTMLMediaElement` interface. */
declare const Audio_2: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<AudioElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "fullscreen" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "loading" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "autoplay" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "play" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "controls" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "destroy" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "loop" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "pause" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "poster" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "volume" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "paused" | "disconnectedCallback" | "render" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "controller" | "screenOrientationController" | "fullscreenController" | "canFullscreen" | "fullscreenOrientation" | "enterFullscreen" | "exitFullscreen" | "mediaElement" | "preload" | "handleDefaultSlotChange" | "startLoadingMedia" | "currentTime" | "muted" | "canLoad" | "logLevel" | "idleDelay" | "attachMediaController" | "store" | "state" | "connectedQueue" | "mediaQueue" | keyof React_2.HTMLAttributes<AudioElement>> & React_2.RefAttributes<AudioElement>>;
export { Audio_2 as Audio }

/** A button for toggling the fullscreen mode of the player.

 💡 The following attributes are applied:

 - `fullscreen`: Applied when the media has entered fullscreen.

 🚨 The `hidden` attribute will be present on this element in the event fullscreen cannot be
 requested (no support). There are default styles for this by setting the `display` property to
 `none`. Important to be aware of this and update it according to your needs. */
export declare const FullscreenButton: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<FullscreenButtonElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "pressed" | "disabled" | "target" | keyof React_2.HTMLAttributes<FullscreenButtonElement>> & React_2.RefAttributes<FullscreenButtonElement>>;

/** This element enables 'actions' to be performed on the media provider based on user gestures.

 The `GestureElement` can be used to build features such as:

 - Click the player to toggle playback.
 - Double-click the player to toggle fullscreen.
 - Tap the sides of the player to seek forwards or backwards.
 - Pause media when the user's mouse leaves the player.

 This is a simple list, but it should give you an idea on when to reach for this element. */
export declare const Gesture: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<GestureElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "type" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "repeat" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "priority" | "action" | "performAction" | keyof React_2.HTMLAttributes<GestureElement>> & React_2.RefAttributes<GestureElement>>;

/** The `<vds-hls>` element adapts the underlying `<video>` element to satisfy the media provider
 contract, which generally involves providing a consistent API for loading, managing, and
 tracking media state.

 This element also introduces support for HLS streaming via the popular `hls.js` library.
 HLS streaming is either [supported natively](https://caniuse.com/?search=hls) (generally
 on iOS), or in environments that [support the Media Stream API](https://caniuse.com/?search=mediastream).

 💡 This element re-dispatches all `hls.js` events so you can listen for them through the
 native DOM interface (i.e., `vds-hls-media-attaching`). */
export declare const Hls: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<HlsElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "fullscreen" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "Hls" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "loading" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "autoplay" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "play" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "controls" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "destroy" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "loop" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "pause" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "poster" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "volume" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "paused" | "disconnectedCallback" | "render" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "controller" | "screenOrientationController" | "fullscreenController" | "canFullscreen" | "fullscreenOrientation" | "enterFullscreen" | "exitFullscreen" | "mediaElement" | "preload" | "handleDefaultSlotChange" | "startLoadingMedia" | "currentTime" | "muted" | "canLoad" | "logLevel" | "idleDelay" | "attachMediaController" | "store" | "state" | "connectedQueue" | "mediaQueue" | "videoElement" | "hlsConfig" | "hlsLibrary" | "hlsEngine" | "isHlsEngineAttached" | "currentHlsSrc" | "isHlsSupported" | "isHlsStream" | "presentationController" | keyof React_2.HTMLAttributes<HlsElement>> & React_2.RefAttributes<HlsElement>>;

/**
 * All media elements exist inside the `<vds-media>` component. It's main jobs are to host the
 * media controller, and expose media state through HTML attributes and CSS properties for styling
 * purposes.
 */
export declare const Media: ForwardRefExoticComponent<Pick<VdsReactComponentProps<MediaElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "provider" | "onVdsMediaProviderConnect" | "fullscreen" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "controller" | "screenOrientationController" | "fullscreenController" | "canFullscreen" | "fullscreenOrientation" | "enterFullscreen" | "exitFullscreen" | "renderOptions" | "connectedCallback" | "disconnectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | keyof HTMLAttributes<MediaElement>> & RefAttributes<MediaElement>>;

export declare const MediaElementContext: Context<MediaElement | null>;

/** This element is responsible for synchronizing elements of the type `MediaProviderElement`.

 Synchronization includes:

 - Single media playback (eg: user plays a video while another is already playing, so we pause
 the newly inactive player).

 - Synchronized media volume (eg: user sets desired volume to 50% on one player, and they expect
 it to be consistent across all players).

 - Saving media volume to local storage (eg: user sets desired to volume 50%, they leave
 the site, and when they come back they expect it to be 50% without any interaction). */
export declare const MediaSync: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<MediaSyncElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "provider" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "singlePlayback" | "syncVolume" | "volumeStorageKey" | "disconnectedCallback" | "render" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | keyof React_2.HTMLAttributes<MediaSyncElement>> & React_2.RefAttributes<MediaSyncElement>>;

/** This element is responsible for managing a `MediaProviderElement` as viewport or page
 visibility changes occur.

 Management includes:

 - Playback or volume changes when page visibility changes (eg: user changes tab or device
 sleeps).

 - Playback or volume changes when viewport visibility changes (eg: user scrolls video in and
 out of view). */
export declare const MediaVisibility: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<MediaVisibilityElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "provider" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "render" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "enterViewport" | "exitViewport" | "viewportEnterDelay" | "enterPage" | "exitPage" | "pageChangeType" | "pageEnterDelay" | "intersectionRoot" | "intersectionThreshold" | "isIntersecting" | keyof React_2.HTMLAttributes<MediaVisibilityElement>> & React_2.RefAttributes<MediaVisibilityElement>>;

/** A button for toggling the muted state of the player.

 💡 The following attributes are applied:

 - `muted`: Applied when media audio has been muted. */
export declare const MuteButton: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<MuteButtonElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "pressed" | "disabled" | keyof React_2.HTMLAttributes<MuteButtonElement>> & React_2.RefAttributes<MuteButtonElement>>;

/** A button for toggling the playback state (play/pause) of the current media.

 💡 The following attributes are applied:

 - `paused`: Applied when media playback has paused. */
export declare const PlayButton: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<PlayButtonElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "pressed" | "disabled" | keyof React_2.HTMLAttributes<PlayButtonElement>> & React_2.RefAttributes<PlayButtonElement>>;

/** Loads and displays the current media poster image. By default, the media provider's
 loading strategy is respected meaning the poster won't load until the media can.

 💡 The following img attributes are applied:

 - `img-loading`: When the poster image is in the process of being downloaded by the browser.
 - `img-loaded`: When the poster image has successfully loaded.
 - `img-error`: When the poster image has failed to load. */
export declare const Poster: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<PosterElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "src" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "alt" | keyof React_2.HTMLAttributes<PosterElement>> & React_2.RefAttributes<PosterElement>>;

/** A custom built `input[type="range"]` that is cross-browser friendly, ARIA friendly, mouse/touch
 friendly and easily stylable. This component allows users to input numeric values between a
 minimum and maximum value.

 💡 The following attributes are available on the host element:

 - `pointing`: Whether a device pointer is within the slider bounds.
 - `dragging`: Whether the slider thumb is currently being dragged.
 - `interactive`: When either `pointing` or `dragging` is true. */
export declare const Slider: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<SliderElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "value" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "step" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "store" | "disabled" | "min" | "max" | "keyboardStep" | "shiftKeyMultiplier" | "customValueText" | "isDragging" | "fillRate" | "fillPercent" | "pointerValue" | "pointerRate" | "pointerPercent" | keyof React_2.HTMLAttributes<SliderElement>> & React_2.RefAttributes<SliderElement>>;

/** Outputs the current slider value as text. */
export declare const SliderValueText: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<SliderValueTextElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "type" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "showHours" | "padHours" | "format" | "decimalPlaces" | keyof React_2.HTMLAttributes<SliderValueTextElement>> & React_2.RefAttributes<SliderValueTextElement>>;

/** Used to load a low-resolution video to be displayed when the user is hovering or dragging
 the slider. The point at which they're hovering or dragging (`pointerValue`) is the preview
 time position. The video will automatically be updated to match, so ensure it's of the same
 length as the original.

 💡 The following attributes are updated for your styling needs:

 - `video-can-play`: Applied when the video is ready for playback.
 - `video-error`: Applied when a media error has been encountered.

 💡 The `canplay` and `error` events are re-dispatched by this element for you to listen to if
 needed. */
export declare const SliderVideo: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<SliderVideoElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "src" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "videoElement" | "willUpdate" | keyof React_2.HTMLAttributes<SliderVideoElement>> & React_2.RefAttributes<SliderVideoElement>>;

/** Outputs a media duration (eg: `currentTime`, `duration`, `bufferedAmount`, etc.) value as time
 formatted text. */
export declare const Time: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<TimeElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "type" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "showHours" | "padHours" | "remainder" | keyof React_2.HTMLAttributes<TimeElement>> & React_2.RefAttributes<TimeElement>>;

/** A slider that lets the user control the current media playback time. */
export declare const TimeSlider: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<TimeSliderElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "value" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "step" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "store" | "disabled" | "min" | "max" | "keyboardStep" | "shiftKeyMultiplier" | "customValueText" | "isDragging" | "fillRate" | "fillPercent" | "pointerValue" | "pointerRate" | "pointerPercent" | "valueText" | "pauseWhileDragging" | "seekingRequestThrottle" | keyof React_2.HTMLAttributes<TimeSliderElement>> & React_2.RefAttributes<TimeSliderElement>>;

/** The foundation for any toggle button such as a `play-button` or `mute-button`. */
export declare const ToggleButton: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<ToggleButtonElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "pressed" | "disabled" | keyof React_2.HTMLAttributes<ToggleButtonElement>> & React_2.RefAttributes<ToggleButtonElement>>;

/**
 * This hook is used to access the current media state on the nearest parent media element (i.e.,
 * `<vds-media>`). Any properties you access here are subscribed to for live updates.
 *
 * @example
 * ```tsx
 * import { useMediaContext } from '@vidstack/player-react';
 *
 * function Component() {
 *  const { playing } = useMediaContext();
 *  return <div>{playing ? 'Media is paused.' : 'Media is playing.'}</div>
 * }
 * ```
 */
export declare function useMediaContext(media?: MediaElement | null | RefObject<MediaElement>): Readonly<MediaContext>;

/**
 * Returns the nearest parent media element (i.e., `<vds-media>`).
 */
export declare function useMediaElement(): MediaElement | null;

/**
 * A media remote provides a simple facade for dispatching media requests to the nearest media
 * controller.
 *
 * @param target - The DOM event target to dispatch request events from.
 *
 * @example
 * ```tsx
 * import { useMediaRemote } from '@vidstack/player-react';
 *
 * function PlayButton() {
 *   const remote = useMediaRemote();
 *   return <button onPointerUp={({ nativeEvent }) => remote.play(nativeEvent)}>Play</button>;
 * }
 * ```
 */
export declare function useMediaRemote(target?: EventTarget | null | RefObject<EventTarget>): MediaRemoteControl;

export declare type VdsElementEventCallbackMap = {
    [Event in keyof VdsElementEventMap as `on${PascalCase<Event>}`]: (event: VdsElementEventMap[Event]) => void;
};

export declare type VdsReactComponentProps<E extends HTMLElement> = Partial<Omit<E, 'children'>> & Partial<VdsElementEventCallbackMap> & React.HTMLAttributes<E> & {
    ref?: React.Ref<E> | undefined;
    children?: React.ReactNode | undefined;
};

/** The `<vds-video>` element adapts the underlying `<video>` element to satisfy the media provider
 contract, which generally involves providing a consistent API for loading, managing, and
 tracking media state.

 Most the logic for this element is contained in the `Html5MediaElement` class because both the
 `<audio>` and `<video>` elements implement the native `HTMLMediaElement` interface. */
export declare const Video: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<VideoElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "fullscreen" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "loading" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "autoplay" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "play" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "controls" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "destroy" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "loop" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "pause" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "poster" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "volume" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "paused" | "disconnectedCallback" | "render" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "controller" | "screenOrientationController" | "fullscreenController" | "canFullscreen" | "fullscreenOrientation" | "enterFullscreen" | "exitFullscreen" | "mediaElement" | "preload" | "handleDefaultSlotChange" | "startLoadingMedia" | "currentTime" | "muted" | "canLoad" | "logLevel" | "idleDelay" | "attachMediaController" | "store" | "state" | "connectedQueue" | "mediaQueue" | "videoElement" | "presentationController" | keyof React_2.HTMLAttributes<VideoElement>> & React_2.RefAttributes<VideoElement>>;

/** A slider control that lets the user specify their desired volume level. */
export declare const VolumeSlider: React_2.ForwardRefExoticComponent<Pick<VdsReactComponentProps<VolumeSliderElement>, "onVdsNoop" | "onVdsContextConsumerConnect" | "onVdsMediaConnect" | "onVdsMediaProviderConnect" | "onVdsFullscreenChange" | "onVdsFullscreenError" | "onVdsFullscreenSupportChange" | "onVdsScreenOrientationChange" | "onVdsScreenOrientationLockChange" | "onVdsLog" | "onVdsHlsLibLoadStart" | "onVdsHlsLibLoaded" | "onVdsHlsLibLoadError" | "onVdsHlsInstance" | "onVdsHlsUnsupported" | "onVdsHlsMediaAttaching" | "onVdsHlsMediaAttached" | "onVdsHlsMediaDetaching" | "onVdsHlsMediaDetached" | "onVdsHlsBufferReset" | "onVdsHlsBufferCodecs" | "onVdsHlsBufferCreated" | "onVdsHlsBufferAppending" | "onVdsHlsBufferAppended" | "onVdsHlsBufferEos" | "onVdsHlsBufferFlushing" | "onVdsHlsBufferFlushed" | "onVdsHlsManifestLoading" | "onVdsHlsManifestLoaded" | "onVdsHlsManifestParsed" | "onVdsHlsLevelSwitching" | "onVdsHlsLevelSwitched" | "onVdsHlsLevelLoading" | "onVdsHlsLevelLoaded" | "onVdsHlsLevelUpdated" | "onVdsHlsLevelPtsUpdated" | "onVdsHlsLevelsUpdated" | "onVdsHlsAudioTracksUpdated" | "onVdsHlsAudioTrackSwitching" | "onVdsHlsAudioTrackSwitched" | "onVdsHlsAudioTrackLoading" | "onVdsHlsAudioTrackLoaded" | "onVdsHlsSubtitleTracksUpdated" | "onVdsHlsSubtitleTracksCleared" | "onVdsHlsSubtitleTrackSwitch" | "onVdsHlsSubtitleTrackLoading" | "onVdsHlsSubtitleTrackLoaded" | "onVdsHlsSubtitleFragProcessed" | "onVdsHlsCuesParsed" | "onVdsHlsNonNativeTextTracksFound" | "onVdsHlsInitPtsFound" | "onVdsHlsFragLoading" | "onVdsHlsFragLoadEmergencyAborted" | "onVdsHlsFragLoaded" | "onVdsHlsFragDecrypted" | "onVdsHlsFragParsingInitSegment" | "onVdsHlsFragParsingUserdata" | "onVdsHlsFragParsingMetadata" | "onVdsHlsFragParsed" | "onVdsHlsFragBufferedData" | "onVdsHlsFragChanged" | "onVdsHlsFpsDrop" | "onVdsHlsFpsDropLevelCapping" | "onVdsHlsError" | "onVdsHlsDestroying" | "onVdsHlsKeyLoading" | "onVdsHlsKeyLoaded" | "onVdsHlsBackBufferReached" | "onVdsAbort" | "onVdsAutoplayChange" | "onVdsAutoplayFail" | "onVdsAutoplay" | "onVdsCanLoad" | "onVdsCanPlayThrough" | "onVdsCanPlay" | "onVdsControlsChange" | "onVdsCurrentSrcChange" | "onVdsDestroy" | "onVdsDurationChange" | "onVdsEmptied" | "onVdsEnd" | "onVdsEnded" | "onVdsError" | "onVdsUserIdleChange" | "onVdsLoadStart" | "onVdsLoadedData" | "onVdsLoadedMetadata" | "onVdsLoopChange" | "onVdsMediaTypeChange" | "onVdsPause" | "onVdsPlayFail" | "onVdsPlay" | "onVdsPlaying" | "onVdsPlaysinlineChange" | "onVdsPosterChange" | "onVdsProgress" | "onVdsReplay" | "onVdsSeeked" | "onVdsSeeking" | "onVdsSrcChange" | "onVdsStalled" | "onVdsStarted" | "onVdsSuspend" | "onVdsTimeUpdate" | "onVdsViewTypeChange" | "onVdsVolumeChange" | "onVdsWaiting" | "onVdsStartLoading" | "onVdsMuteRequest" | "onVdsUnmuteRequest" | "onVdsEnterFullscreenRequest" | "onVdsExitFullscreenRequest" | "onVdsPlayRequest" | "onVdsPauseRequest" | "onVdsSeekRequest" | "onVdsSeekingRequest" | "onVdsVolumeChangeRequest" | "onVdsResumeUserIdleRequest" | "onVdsPauseUserIdleRequest" | "onVdsShowPosterRequest" | "onVdsHidePosterRequest" | "onVdsLoopRequest" | "onVdsMediaVisibilityChange" | "onVdsMediaVolumeSync" | "onVdsSliderDragStart" | "onVdsSliderDragEnd" | "value" | "onVdsSliderValueChange" | "onVdsSliderDragValueChange" | "onVdsSliderPointerValueChange" | "onVdsVideoPresentationChange" | "accessKeyLabel" | "autocapitalize" | "innerText" | "offsetHeight" | "offsetLeft" | "offsetParent" | "offsetTop" | "offsetWidth" | "outerText" | "spellcheck" | "attachInternals" | "click" | "addEventListener" | "removeEventListener" | "attributes" | "classList" | "clientHeight" | "clientLeft" | "clientTop" | "clientWidth" | "localName" | "namespaceURI" | "onfullscreenchange" | "onfullscreenerror" | "outerHTML" | "ownerDocument" | "part" | "scrollHeight" | "scrollLeft" | "scrollTop" | "scrollWidth" | "shadowRoot" | "tagName" | "attachShadow" | "closest" | "getAttribute" | "getAttributeNS" | "getAttributeNames" | "getAttributeNode" | "getAttributeNodeNS" | "getBoundingClientRect" | "getClientRects" | "getElementsByClassName" | "getElementsByTagName" | "getElementsByTagNameNS" | "hasAttribute" | "hasAttributeNS" | "hasAttributes" | "hasPointerCapture" | "insertAdjacentElement" | "insertAdjacentHTML" | "insertAdjacentText" | "matches" | "releasePointerCapture" | "removeAttribute" | "removeAttributeNS" | "removeAttributeNode" | "requestFullscreen" | "requestPointerLock" | "scroll" | "scrollBy" | "scrollIntoView" | "scrollTo" | "setAttribute" | "setAttributeNS" | "setAttributeNode" | "setAttributeNodeNS" | "setPointerCapture" | "toggleAttribute" | "webkitMatchesSelector" | "baseURI" | "childNodes" | "firstChild" | "isConnected" | "lastChild" | "nextSibling" | "nodeName" | "nodeType" | "nodeValue" | "parentElement" | "parentNode" | "previousSibling" | "textContent" | "appendChild" | "cloneNode" | "compareDocumentPosition" | "contains" | "getRootNode" | "hasChildNodes" | "insertBefore" | "isDefaultNamespace" | "isEqualNode" | "isSameNode" | "lookupNamespaceURI" | "lookupPrefix" | "normalize" | "removeChild" | "replaceChild" | "ATTRIBUTE_NODE" | "CDATA_SECTION_NODE" | "COMMENT_NODE" | "DOCUMENT_FRAGMENT_NODE" | "DOCUMENT_NODE" | "DOCUMENT_POSITION_CONTAINED_BY" | "DOCUMENT_POSITION_CONTAINS" | "DOCUMENT_POSITION_DISCONNECTED" | "DOCUMENT_POSITION_FOLLOWING" | "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC" | "DOCUMENT_POSITION_PRECEDING" | "DOCUMENT_TYPE_NODE" | "ELEMENT_NODE" | "ENTITY_NODE" | "ENTITY_REFERENCE_NODE" | "NOTATION_NODE" | "PROCESSING_INSTRUCTION_NODE" | "TEXT_NODE" | "dispatchEvent" | "ariaAtomic" | "ariaAutoComplete" | "ariaBusy" | "ariaChecked" | "ariaColCount" | "ariaColIndex" | "ariaColSpan" | "ariaCurrent" | "ariaDisabled" | "ariaExpanded" | "ariaHasPopup" | "ariaHidden" | "ariaKeyShortcuts" | "ariaLabel" | "ariaLevel" | "ariaLive" | "ariaModal" | "ariaMultiLine" | "ariaMultiSelectable" | "ariaOrientation" | "ariaPlaceholder" | "ariaPosInSet" | "ariaPressed" | "ariaReadOnly" | "ariaRequired" | "ariaRoleDescription" | "ariaRowCount" | "ariaRowIndex" | "ariaRowSpan" | "ariaSelected" | "ariaSetSize" | "ariaSort" | "ariaValueMax" | "ariaValueMin" | "ariaValueNow" | "ariaValueText" | "animate" | "getAnimations" | "after" | "before" | "remove" | "replaceWith" | "innerHTML" | "nextElementSibling" | "previousElementSibling" | "childElementCount" | "firstElementChild" | "lastElementChild" | "append" | "prepend" | "querySelector" | "querySelectorAll" | "replaceChildren" | "assignedSlot" | "oncopy" | "oncut" | "onpaste" | "enterKeyHint" | "isContentEditable" | "onabort" | "onanimationcancel" | "onanimationend" | "onanimationiteration" | "onanimationstart" | "onauxclick" | "onblur" | "oncanplay" | "oncanplaythrough" | "onchange" | "onclick" | "onclose" | "oncontextmenu" | "oncuechange" | "ondblclick" | "ondrag" | "ondragend" | "ondragenter" | "ondragleave" | "ondragover" | "ondragstart" | "ondrop" | "ondurationchange" | "onemptied" | "onended" | "onerror" | "onfocus" | "onformdata" | "ongotpointercapture" | "oninput" | "oninvalid" | "onkeydown" | "onkeypress" | "onkeyup" | "onload" | "onloadeddata" | "onloadedmetadata" | "onloadstart" | "onlostpointercapture" | "onmousedown" | "onmouseenter" | "onmouseleave" | "onmousemove" | "onmouseout" | "onmouseover" | "onmouseup" | "onpause" | "onplay" | "onplaying" | "onpointercancel" | "onpointerdown" | "onpointerenter" | "onpointerleave" | "onpointermove" | "onpointerout" | "onpointerover" | "onpointerup" | "onprogress" | "onratechange" | "onreset" | "onresize" | "onscroll" | "onsecuritypolicyviolation" | "onseeked" | "onseeking" | "onselect" | "onselectionchange" | "onselectstart" | "onslotchange" | "onstalled" | "onsubmit" | "onsuspend" | "ontimeupdate" | "ontoggle" | "ontouchcancel" | "ontouchend" | "ontouchmove" | "ontouchstart" | "ontransitioncancel" | "ontransitionend" | "ontransitionrun" | "ontransitionstart" | "onvolumechange" | "onwaiting" | "onwebkitanimationend" | "onwebkitanimationiteration" | "onwebkitanimationstart" | "onwebkittransitionend" | "onwheel" | "autofocus" | "dataset" | "nonce" | "blur" | "focus" | "step" | "disconnectedCallback" | "renderOptions" | "connectedCallback" | "renderRoot" | "isUpdatePending" | "hasUpdated" | "addController" | "removeController" | "attributeChangedCallback" | "requestUpdate" | "updateComplete" | "store" | "disabled" | "min" | "max" | "keyboardStep" | "shiftKeyMultiplier" | "customValueText" | "isDragging" | "fillRate" | "fillPercent" | "pointerValue" | "pointerRate" | "pointerPercent" | keyof React_2.HTMLAttributes<VolumeSliderElement>> & React_2.RefAttributes<VolumeSliderElement>>;

export { }
